#undef __NO_VERSION__
#include <linux/version.h>

#include <linux/string.h>
#include <linux/errno.h>
#ifdef MODULE
   #if defined(CONFIG_MODVERSIONS) && !defined(MODVERSIONS)
      #define MODVERSIONS
   #endif

   #ifdef MODVERSIONS
      #include <config/modversions.h>
   #endif

   #include <linux/module.h>
#endif

#include "nikal.h"

#ifdef MODULE_LICENSE
   #ifdef nNIKAL100_kCopyrightString
      MODULE_LICENSE(nNIKAL100_kCopyrightString);
   #endif
#endif

#ifdef MODULE_AUTHOR
   #ifdef nNIKAL100_kAuthorString
      MODULE_AUTHOR(nNIKAL100_kAuthorString);
   #endif
#endif

#ifdef MODULE_DESCRIPTION
   #ifdef nNIKAL100_kDescriptionString
      MODULE_DESCRIPTION(nNIKAL100_kDescriptionString);
   #endif
#endif


#include <linux/pci.h>
#include <linux/mod_devicetable.h>

int nNIAPAL200_driverEntry(struct nNIKAL100_tDriver *driver);

static struct nNIKAL100_tDriver atomicDriver;
static void* nativeDriverRegistration;

/* native driver registration functions. These functions are auto-generated */
void *registerNativeDriver(nNIKAL100_tDriver*);
void unregisterNativeDriver(void*);

int init_module(void)
{
   atomicDriver.module = THIS_MODULE;

   if (nNIAPAL200_driverEntry(&atomicDriver))
      return -ENODEV;

   nativeDriverRegistration = registerNativeDriver(&atomicDriver);

   if (nativeDriverRegistration == NULL)
   {
      if (atomicDriver.unload)
         atomicDriver.unload(&atomicDriver);
      return -ENODEV;
   }

   return 0;
}

void cleanup_module(void)
{
   unregisterNativeDriver(nativeDriverRegistration);

   if (atomicDriver.unload)
      atomicDriver.unload(&atomicDriver);
}

static struct pci_device_id nipciidtable[] = {
   {0x8086, 0x24C3, 0x8086, 0x24C0, 0, 0, (unsigned long)&atomicDriver},
   {0x8086, 0x266A, 0x8086, 0x1999, 0, 0, (unsigned long)&atomicDriver},
   {0x8086, 0x27DA, 0x1093, 0x71F9, 0, 0, (unsigned long)&atomicDriver},
   {0x8086, 0x27DA, 0x1093, 0x7309, 0, 0, (unsigned long)&atomicDriver},
   {0x8086, 0x27DA, 0x1093, 0xFE70, 0, 0, (unsigned long)&atomicDriver},
   {0x8086, 0x2930, 0x1093, 0x7321, 0, 0, (unsigned long)&atomicDriver},
   {0x8086, 0x2930, 0x1093, 0x748F, 0, 0, (unsigned long)&atomicDriver},
   {0x8086, 0x266A, 0x1093, 0x71FB, 0, 0, (unsigned long)&atomicDriver},
   {0x8086, 0x27DA, 0x1093, 0x71FA, 0, 0, (unsigned long)&atomicDriver},
   {0x8086, 0x27DA, 0x1093, 0x7294, 0, 0, (unsigned long)&atomicDriver},
   {0x8086, 0x3B30, 0x1093, 0x74CC, 0, 0, (unsigned long)&atomicDriver},
   {0x8086, 0x3B30, 0x1093, 0x753D, 0, 0, (unsigned long)&atomicDriver},
   {0x1093, 0x727E, PCI_ANY_ID, PCI_ANY_ID, 0, 0, (unsigned long)&atomicDriver},
   {0x10DE, 0x368, 0x1093, 0x7313, 0, 0, (unsigned long)&atomicDriver},
   {0x8086, 0x3B30, 0x1093, 0x754C, 0, 0, (unsigned long)&atomicDriver},
   {0x8086, 0x283E, 0x1093, 0x7546, 0, 0, (unsigned long)&atomicDriver},
   {0x8086, 0x1C22, 0x1093, 0x75D9, 0, 0, (unsigned long)&atomicDriver},
   {0x8086, 0x1C22, 0x1093, 0x75B9, 0, 0, (unsigned long)&atomicDriver},
   {0x8086, 0x1E22, 0x1093, 0x75F4, 0, 0, (unsigned long)&atomicDriver},
   {0x8086, 0x8C22, 0x1093, 0x770A, 0, 0, (unsigned long)&atomicDriver},
   {0x8086, 0x8D22, 0x1093, 0x77A4, 0, 0, (unsigned long)&atomicDriver},
   {0x8086, 0xA123, 0x1093, 0x77F2, 0, 0, (unsigned long)&atomicDriver},
   {0x8086, 0xA2A3, 0x1093, 0x79D2, 0, 0, (unsigned long)&atomicDriver},
   {0x8086, 0x43A3, 0x1093, 0x79D2, 0, 0, (unsigned long)&atomicDriver},
    { },
};
MODULE_DEVICE_TABLE(pci, nipciidtable);

static struct pci_driver niPciDriver = {
   .name = "nismbus",
   .id_table = nipciidtable,
};

void *registerNativeDriver(nNIKAL100_tDriver* ignore)
{
   return nNIKAL200_registerPCIDriver(&niPciDriver);
}
void unregisterNativeDriver(void* data)
{
   nNIKAL200_unregisterPCIDriver(data);
}


#define nNIKAL100_mExportSymbol EXPORT_SYMBOL
